@@ -1,6 +1,7 @@
-from sympy import (Symbol, S, exp, log, sqrt, oo, E, zoo, tan,
-        sin, pi)
-from sympy.calculus.util import not_empty_in, AccumBounds
+from sympy import (Symbol, S, exp, log, sqrt, oo, E, zoo, pi, tan, sin, cos,
+                   cot, sec, csc, Abs)
+from sympy.calculus.util import (function_range, continuous_domain, not_empty_in,
+                                 periodicity, lcim, AccumBounds)
 from sympy.core import Add, Mul, Pow
 from sympy.sets.sets import Interval, FiniteSet, Complement, Union
 from sympy.utilities.pytest import raises
@@ -9,6 +10,32 @@ from sympy.abc import x
 a = Symbol('a', real=True)
 
 
+def test_function_range():
+    x = Symbol('x')
+    assert function_range(sin(x), x, Interval(-pi/2, pi/2)) == Interval(-1, 1)
+    assert function_range(sin(x), x, Interval(0, pi)) == Interval(0, 1)
+    assert function_range(tan(x), x, Interval(0, pi)) == Interval(-oo, oo)
+    assert function_range(tan(x), x, Interval(pi/2, pi)) == Interval(-oo, 0)
+    assert function_range((x + 3)/(x - 2), x, Interval(-5, 5)) == Interval(-oo, oo)
+    assert function_range(1/(x**2), x, Interval(-1, 1)) == Interval(1, oo)
+    assert function_range(exp(x), x, Interval(-1, 1)) == Interval(exp(-1), exp(1))
+    assert function_range(log(x) - x, x, S.Reals) == Interval(-oo, -1)
+    assert function_range(sqrt(3*x - 1), x, Interval(0, 2)) == Interval(0, sqrt(5))
+
+
+def test_continuous_domain():
+    x = Symbol('x')
+    assert continuous_domain(sin(x), x, Interval(0, 2*pi)) == Interval(0, 2*pi)
+    assert continuous_domain(tan(x), x, Interval(0, 2*pi)) == \
+        Union(Interval(0, pi/2, False, True), Interval(pi/2, 3*pi/2, True, True),
+              Interval(3*pi/2, 2*pi, True, False))
+    assert continuous_domain((x - 1)/((x - 1)**2), x, S.Reals) == \
+        Union(Interval(-oo, 1, True, True), Interval(1, oo, True, True))
+    assert continuous_domain(log(x) + log(4*x - 1), x, S.Reals) == \
+        Interval(1/4, oo, True, True)
+    assert continuous_domain(1/sqrt(x - 3), x, S.Reals) == Interval(3, oo, True, True)
+
+
 def test_not_empty_in():
     assert not_empty_in(FiniteSet(x, 2*x).intersect(Interval(1, 2, True, False)), x) == \
         Interval(S(1)/2, 2, True, False)
@@ -36,6 +63,62 @@ def test_not_empty_in():
         Union(Interval(-2, -1, True, False), Interval(2, oo))
 
 
+def test_periodicity():
+    x = Symbol('x')
+    y = Symbol('y')
+
+    assert periodicity(sin(2*x), x) == pi
+    assert periodicity((-2)*tan(4*x), x) == pi/4
+    assert periodicity(sin(x)**2, x) == 2*pi
+    assert periodicity(3**tan(3*x), x) == pi/3
+    assert periodicity(tan(x)*cos(x), x) == 2*pi
+    assert periodicity(sin(x)**(tan(x)), x) == 2*pi
+    assert periodicity(tan(x)*sec(x), x) == 2*pi
+    assert periodicity(sin(2*x)*cos(2*x) - y, x) == pi/2
+    assert periodicity(tan(x) + cot(x), x) == pi
+    assert periodicity(sin(x) - cos(2*x), x) == 2*pi
+    assert periodicity(sin(x) - 1, x) == 2*pi
+    assert periodicity(sin(4*x) + sin(x)*cos(x), x) == pi
+    assert periodicity(exp(sin(x)), x) == 2*pi
+    assert periodicity(log(cot(2*x)) - sin(cos(2*x)), x) == pi
+    assert periodicity(sin(2*x)*exp(tan(x) - csc(2*x)), x) == pi
+    assert periodicity(cos(sec(x) - csc(2*x)), x) == 2*pi
+    assert periodicity(tan(sin(2*x)), x) == pi
+    assert periodicity(2*tan(x)**2, x) == pi
+
+    assert periodicity(sin(x)**2 + cos(x)**2, x) == S.Zero
+    assert periodicity(tan(x), y) == S.Zero
+
+    assert periodicity(exp(x), x) is None
+    assert periodicity(log(x), x) is None
+    assert periodicity(exp(x)**sin(x), x) is None
+    assert periodicity(sin(x)**y, y) is None
+
+    assert periodicity(x**3 - x**2 + 1, x) is None
+    assert periodicity(Abs(x), x) is None
+    assert periodicity(Abs(x**2 - 1), x) is None
+
+
+def test_periodicity_check():
+    x = Symbol('x')
+    y = Symbol('y')
+
+    assert periodicity(tan(x), x, check=True) == pi
+    assert periodicity(sin(x) + cos(x), x, check=True) == 2*pi
+    raises(NotImplementedError, lambda: periodicity(sec(x), x, check=True))
+    raises(NotImplementedError, lambda: periodicity(sin(x*y), x, check=True))
+
+
+def test_lcim():
+    from sympy import pi
+
+    assert lcim([S(1)/2, S(2), S(3)]) == 6
+    assert lcim([pi/2, pi/4, pi]) == pi
+    assert lcim([2*pi, pi/2]) == 2*pi
+    assert lcim([S(1), 2*pi]) is None
+    assert lcim([S(2) + 2*E, E/3 + S(1)/3, S(1) + E]) == S(2) + 2*E
+
+
 def test_AccumBounds():
     assert AccumBounds(1, 2).args == (1, 2)
     assert AccumBounds(1, 2).delta == S(1)
