@@ -5,7 +5,7 @@ from sympy.concrete.expr_with_intlimits import ExprWithIntLimits
 from sympy.core.function import Derivative
 from sympy.core.relational import Eq
 from sympy.core.singleton import S
-from sympy.core.symbol import Dummy, Wild
+from sympy.core.symbol import Dummy, Wild, Symbol
 from sympy.core.add import Add
 from sympy.calculus.singularities import is_decreasing
 from sympy.concrete.gosper import gosper_sum
@@ -13,11 +13,12 @@ from sympy.functions.special.zeta_functions import zeta
 from sympy.functions.elementary.piecewise import Piecewise
 from sympy.logic.boolalg import And
 from sympy.polys import apart, PolynomialError
-from sympy.solvers import solve
 from sympy.series.limits import limit
 from sympy.series.order import O
+from sympy.sets.sets import FiniteSet
+from sympy.solvers import solve
+from sympy.solvers.solveset import solveset
 from sympy.core.compatibility import range
-from sympy.tensor.indexed import Idx
 
 
 class Sum(AddWithLimits, ExprWithIntLimits):
@@ -183,8 +184,6 @@ class Sum(AddWithLimits, ExprWithIntLimits):
             if dif.is_integer and (dif < 0) == True:
                 a, b = b + 1, a - 1
                 f = -f
-            if isinstance(i, Idx):
-                i = i.label
 
             newf = eval_sum(f, (i, a, b))
             if newf is None:
@@ -235,7 +234,7 @@ class Sum(AddWithLimits, ExprWithIntLimits):
         # don't want to differentiate wrt any free symbol in the upper or lower
         # limits
         # XXX remove this test for free_symbols when the default _eval_derivative is in
-        if x not in self.free_symbols:
+        if isinstance(x, Symbol) and x not in self.free_symbols:
             return S.Zero
 
         # get limits and the function
@@ -252,8 +251,6 @@ class Sum(AddWithLimits, ExprWithIntLimits):
                 return None
             df = Derivative(f, x, evaluate=True)
             rv = self.func(df, limit)
-            if limit[0] not in df.free_symbols:
-                rv = rv.doit()
             return rv
         else:
             return NotImplementedError('Lower and upper bound expected.')
@@ -269,9 +266,41 @@ class Sum(AddWithLimits, ExprWithIntLimits):
 
         return Sum(f, (k, upper + 1, new_upper)).doit()
 
-    def _eval_simplify(self, ratio, measure):
-        from sympy.simplify.simplify import sum_simplify
-        return sum_simplify(self)
+    def _eval_simplify(self, ratio=1.7, measure=None):
+        from sympy.simplify.simplify import factor_sum, sum_combine
+        from sympy.core.function import expand
+        from sympy.core.mul import Mul
+
+        # split the function into adds
+        terms = Add.make_args(expand(self.function))
+        s_t = [] # Sum Terms
+        o_t = [] # Other Terms
+
+        for term in terms:
+            if term.has(Sum):
+                # if there is an embedded sum here
+                # it is of the form x * (Sum(whatever))
+                # hence we make a Mul out of it, and simplify all interior sum terms
+                subterms = Mul.make_args(expand(term))
+                out_terms = []
+                for subterm in subterms:
+                    # go through each term
+                    if isinstance(subterm, Sum):
+                        # if it's a sum, simplify it
+                        out_terms.append(subterm._eval_simplify())
+                    else:
+                        # otherwise, add it as is
+                        out_terms.append(subterm)
+
+                # turn it back into a Mul
+                s_t.append(Mul(*out_terms))
+            else:
+                o_t.append(term)
+
+        # next try to combine any interior sums for further simplification
+        result = Add(sum_combine(s_t), *o_t)
+
+        return factor_sum(result, limits=self.limits)
 
     def _eval_summation(self, f, x):
         return None
@@ -285,7 +314,8 @@ class Sum(AddWithLimits, ExprWithIntLimits):
         First Part:
         One part is the question whether all the terms are well defined, i.e.,
         they are finite in a sum and also non-zero in a product. Zero
-        is the analogy of (minus) infinity in products as :math:`e^{-\infty} = 0`.
+        is the analogy of (minus) infinity in products as
+        :math:`e^{-\infty} = 0`.
 
         Second Part:
         The second part is the question of convergence after infinities,
@@ -355,8 +385,8 @@ class Sum(AddWithLimits, ExprWithIntLimits):
         sequence_term = self.function
 
         if len(sequence_term.free_symbols) > 1:
-            raise NotImplementedError("convergence checking for more that one symbol "
-                                        "containing series is not handled")
+            raise NotImplementedError("convergence checking for more than one symbol "
+                                      "containing series is not handled")
 
         if lower_limit.is_finite and upper_limit.is_finite:
             return S.true
@@ -454,14 +484,22 @@ class Sum(AddWithLimits, ExprWithIntLimits):
             return S.false
 
         ### ------------- integral test -------------- ###
-        if is_decreasing(sequence_term, interval):
-            integral_val = Integral(sequence_term, (sym, lower_limit, upper_limit))
-            try:
-                integral_val_evaluated = integral_val.doit()
-                if integral_val_evaluated.is_number:
-                    return S(integral_val_evaluated.is_finite)
-            except NotImplementedError:
-                pass
+        maxima = solveset(sequence_term.diff(sym), sym, interval)
+        if not maxima:
+            check_interval = interval
+        elif isinstance(maxima, FiniteSet) and maxima.sup.is_number:
+            check_interval = Interval(maxima.sup, interval.sup)
+            if (
+                    is_decreasing(sequence_term, check_interval) or
+                    is_decreasing(-sequence_term, check_interval)):
+                integral_val = Integral(
+                    sequence_term, (sym, lower_limit, upper_limit))
+                try:
+                    integral_val_evaluated = integral_val.doit()
+                    if integral_val_evaluated.is_number:
+                        return S(integral_val_evaluated.is_finite)
+                except NotImplementedError:
+                    pass
 
         ### -------------- Dirichlet tests -------------- ###
         if order.expr.is_Mul:
@@ -487,7 +525,7 @@ class Sum(AddWithLimits, ExprWithIntLimits):
                     return dirich2
 
         raise NotImplementedError("The algorithm to find the Sum convergence of %s "
-                                    "is not yet implemented" % (sequence_term))
+                                  "is not yet implemented" % (sequence_term))
 
     def is_absolutely_convergent(self):
         """
@@ -572,7 +610,7 @@ class Sum(AddWithLimits, ExprWithIntLimits):
         i, a, b = self.limits[0]
         if (a > b) == True:
             if a - b == 1:
-                return S.Zero,S.Zero
+                return S.Zero, S.Zero
             a, b = b + 1, a - 1
             f = -f
         s = S.Zero
@@ -959,6 +997,7 @@ def _eval_sum_hyper(f, i, a):
     from sympy.functions import hyper
     from sympy.simplify import hyperexpand, hypersimp, fraction, simplify
     from sympy.polys.polytools import Poly, factor
+    from sympy.core.numbers import Float
 
     if a != 0:
         return _eval_sum_hyper(f.subs(i, i + a), i, 0)
@@ -972,6 +1011,10 @@ def _eval_sum_hyper(f, i, a):
     if hs is None:
         return None
 
+    if isinstance(hs, Float):
+        from sympy.simplify.simplify import nsimplify
+        hs = nsimplify(hs)
+
     numer, denom = fraction(factor(hs))
     top, topl = numer.as_coeff_mul(i)
     bot, botl = denom.as_coeff_mul(i)
