@@ -5,11 +5,10 @@ from .compatibility import ordered
 from .expr import Expr
 from .evalf import EvalfMixin
 from .function import _coeff_isneg
-from .symbol import Symbol
 from .sympify import _sympify
 from .evaluate import global_evaluate
 
-from sympy.logic.boolalg import Boolean
+from sympy.logic.boolalg import Boolean, BooleanAtom
 
 __all__ = (
     'Rel', 'Eq', 'Ne', 'Lt', 'Le', 'Gt', 'Ge',
@@ -116,7 +115,7 @@ class Relational(Boolean, Expr, EvalfMixin):
         elif r.func in (Eq, Ne):
             r = r.func(*ordered(r.args), evaluate=False)
         else:
-            raise NotImplemented
+            raise NotImplementedError
         if r.lhs.is_Number and not r.rhs.is_Number:
             r = r.reversed
         elif r.rhs.is_Symbol and not r.lhs.is_Symbol:
@@ -137,14 +136,14 @@ class Relational(Boolean, Expr, EvalfMixin):
             if a.func in (Eq, Ne) or b.func in (Eq, Ne):
                 if a.func != b.func:
                     return False
-                l = a.lhs.equals(b.lhs, failing_expression=failing_expression)
-                r = a.rhs.equals(b.rhs, failing_expression=failing_expression)
+                l, r = [i.equals(j, failing_expression=failing_expression)
+                    for i, j in zip(a.args, b.args)]
                 if l is True:
                     return r
                 if r is True:
                     return l
-                lr = a.lhs.equals(b.rhs, failing_expression=failing_expression)
-                rl = a.rhs.equals(b.lhs, failing_expression=failing_expression)
+                lr, rl = [i.equals(j, failing_expression=failing_expression)
+                    for i, j in zip(a.args, b.reversed.args)]
                 if lr is True:
                     return rl
                 if rl is True:
@@ -172,25 +171,19 @@ class Relational(Boolean, Expr, EvalfMixin):
 
     def _eval_simplify(self, ratio, measure):
         r = self
-        r = r.func(r.lhs.simplify(ratio=ratio, measure=measure),
-                   r.rhs.simplify(ratio=ratio, measure=measure))
-        if r not in (S.true, S.false):
-            if isinstance(r.lhs, Expr) and isinstance(r.rhs, Expr):
-                dif = r.lhs - r.rhs
-                # We want a Number to compare with zero and be sure to get a
-                # True/False answer.  Check if we can deduce that dif is
-                # definitively zero or non-zero.  If non-zero, replace with an
-                # approximation.  If .equals(0) gives None, cannot be deduced.
-                if not dif.has(Symbol):
-                    know = dif.equals(0)
-                    if know == True:
-                        dif = S.Zero
-                    elif know == False:
-                        dif = dif.evalf()
-                # Can definitively compare a Number to zero, if appropriate.
-                if dif.is_Number and (dif.is_real or r.func in (Eq, Ne)):
-                    # Always T/F (we never return an expression w/ the evalf)
-                    r = r.func._eval_relation(dif, S.Zero)
+        r = r.func(*[i.simplify(ratio=ratio, measure=measure)
+            for i in r.args])
+        if r.is_Relational:
+            dif = r.lhs - r.rhs
+            # replace dif with a valid Number that will
+            # allow a definitive comparison with 0
+            v = None
+            if dif.is_comparable:
+                v = dif.n(2)
+            elif dif.equals(0):  # XXX this is expensive
+                v = S.Zero
+            if v is not None:
+                r = r.func._eval_relation(v, S.Zero)
 
         r = r.canonical
         if measure(r) < ratio*measure(self):
@@ -212,8 +205,8 @@ class Relational(Boolean, Expr, EvalfMixin):
 
         >>> from sympy import Symbol, Eq
         >>> x = Symbol('x', real=True)
-        >>> (x>0).as_set()
-        (0, oo)
+        >>> (x > 0).as_set()
+        Interval.open(0, oo)
         >>> Eq(x, 0).as_set()
         {0}
 
@@ -291,6 +284,11 @@ class Equality(Relational):
     is_Equality = True
 
     def __new__(cls, lhs, rhs=0, **options):
+        from sympy.core.add import Add
+        from sympy.core.logic import fuzzy_bool
+        from sympy.core.expr import _n2
+        from sympy.simplify.simplify import clear_coefficients
+
         lhs = _sympify(lhs)
         rhs = _sympify(rhs)
 
@@ -309,13 +307,57 @@ class Equality(Relational):
             # If expressions have the same structure, they must be equal.
             if lhs == rhs:
                 return S.true
+            elif all(isinstance(i, BooleanAtom) for i in (rhs, lhs)):
+                return S.false
 
-            # If appropriate, check if the difference evaluates.  Detect
-            # incompatibility such as lhs real and rhs not real.
-            if lhs.is_complex and rhs.is_complex:
-                r = (lhs - rhs).is_zero
-                if r is not None:
-                    return _sympify(r)
+            # check finiteness
+            fin = L, R = [i.is_finite for i in (lhs, rhs)]
+            if None not in fin:
+                if L != R:
+                    return S.false
+                if L is False:
+                    if lhs == -rhs:  # Eq(oo, -oo)
+                        return S.false
+                    return S.true
+            elif None in fin and False in fin:
+                return Relational.__new__(cls, lhs, rhs, **options)
+
+            if all(isinstance(i, Expr) for i in (lhs, rhs)):
+                # see if the difference evaluates
+                dif = lhs - rhs
+                z = dif.is_zero
+                if z is not None:
+                    if z is False and dif.is_commutative:  # issue 10728
+                        return S.false
+                    if z:
+                        return S.true
+                # evaluate numerically if possible
+                n2 = _n2(lhs, rhs)
+                if n2 is not None:
+                    return _sympify(n2 == 0)
+                # see if the ratio evaluates
+                n, d = dif.as_numer_denom()
+                rv = None
+                if n.is_zero:
+                    rv = d.is_nonzero
+                elif n.is_finite:
+                    if d.is_infinite:
+                        rv = S.true
+                    elif n.is_zero is False:
+                        rv = d.is_infinite
+                        if rv is None:
+                            # if the condition that makes the denominator infinite does not
+                            # make the original expression True then False can be returned
+                            l, r = clear_coefficients(d, S.Infinity)
+                            args = [_.subs(l, r) for _ in (lhs, rhs)]
+                            if args != [lhs, rhs]:
+                                rv = fuzzy_bool(Eq(*args))
+                                if rv is True:
+                                    rv = None
+                elif any(a.is_infinite for a in Add.make_args(n)):  # (inf or nan)/x != 0
+                    rv = S.false
+                if rv is not None:
+                    return _sympify(rv)
 
         return Relational.__new__(cls, lhs, rhs, **options)
 
@@ -368,7 +410,7 @@ class Unequality(Relational):
 
         if evaluate:
             is_equal = Equality(lhs, rhs)
-            if is_equal == True or is_equal == False:
+            if isinstance(is_equal, BooleanAtom):
                 return ~is_equal
 
         return Relational.__new__(cls, lhs, rhs, **options)
@@ -636,7 +678,7 @@ class GreaterThan(_Greater):
     >>> type( e )
     And
     >>> e
-    And(x < y, y < z)
+    (x < y) & (y < z)
 
     Note that this is different than chaining an equality directly via use of
     parenthesis (this is currently an open bug in SymPy [2]_):
