@@ -205,7 +205,7 @@ class exp(ExpBase):
         else:
             raise ArgumentIndexError(self, argindex)
 
-    def _eval_refine(self):
+    def _eval_refine(self, assumptions):
         from sympy.assumptions import ask, Q
         arg = self.args[0]
         if arg.is_Mul:
@@ -454,7 +454,7 @@ class exp(ExpBase):
 
 
 class log(Function):
-    """
+    r"""
     The natural logarithm function `\ln(x)` or `\log(x)`.
     Logarithms are taken with the natural base, `e`. To get
     a logarithm of a different base ``b``, use ``log(x, b)``,
@@ -472,13 +472,11 @@ class log(Function):
         """
         if argindex == 1:
             return 1/self.args[0]
-            s = Dummy('x')
-            return Lambda(s**(-1), s)
         else:
             raise ArgumentIndexError(self, argindex)
 
     def inverse(self, argindex=1):
-        """
+        r"""
         Returns `e^x`, the inverse function of `\log(x)`.
         """
         return exp
@@ -572,7 +570,7 @@ class log(Function):
     @staticmethod
     @cacheit
     def taylor_term(n, x, *previous_terms):  # of log(1+x)
-        """
+        r"""
         Returns the next term in the Taylor series expansion of `\log(1+x)`.
         """
         from sympy import powsimp
@@ -752,7 +750,7 @@ class log(Function):
 
 
 class LambertW(Function):
-    """
+    r"""
     The Lambert W function `W(z)` is defined as the inverse
     function of `w \exp(w)` [1]_.
 
