@@ -3,15 +3,17 @@ from __future__ import print_function, division
 from sympy.core.add import Add
 from sympy.core.basic import sympify, cacheit
 from sympy.core.function import Function, ArgumentIndexError
-from sympy.core.numbers import igcdex, Rational
+from sympy.core.numbers import igcdex, Rational, pi
 from sympy.core.singleton import S
-from sympy.core.symbol import Symbol
+from sympy.core.symbol import Symbol, Wild
 from sympy.core.logic import fuzzy_not
 from sympy.functions.combinatorial.factorials import factorial, RisingFactorial
-from sympy.functions.elementary.miscellaneous import sqrt
+from sympy.functions.elementary.miscellaneous import sqrt, Min, Max
 from sympy.functions.elementary.exponential import log, exp
+from sympy.functions.elementary.integers import floor
 from sympy.functions.elementary.hyperbolic import (acoth, asinh, atanh, cosh,
     coth, HyperbolicFunction, sinh, tanh)
+from sympy.sets.sets import FiniteSet
 from sympy.utilities.iterables import numbered_symbols
 from sympy.core.compatibility import range
 
@@ -61,6 +63,32 @@ class TrigonometricFunction(Function):
             re, im = self.args[0].as_real_imag()
         return (re, im)
 
+    def _period(self, general_period, symbol=None):
+        f = self.args[0]
+        if symbol is None:
+            symbol = tuple(f.free_symbols)[0]
+
+        if not f.has(symbol):
+            return S.Zero
+
+        if f == symbol:
+            return general_period
+
+        if symbol in f.free_symbols:
+            p, q = Wild('p'), Wild('q')
+            if f.is_Mul:
+                g, h = f.as_independent(symbol)
+                if h == symbol:
+                    return general_period/abs(g)
+
+            if f.is_Add:
+                a, h = f.as_independent(symbol)
+                g, h = h.as_independent(symbol, as_Add=False)
+                if h == symbol:
+                    return general_period/abs(g)
+
+        raise NotImplementedError("Use the periodicity function instead.")
+
 
 def _peeloff_pi(arg):
     """
@@ -214,6 +242,9 @@ class sin(TrigonometricFunction):
     .. [4] http://mathworld.wolfram.com/TrigonometryAngles.html
     """
 
+    def period(self, symbol=None):
+        return self._period(2*pi, symbol)
+
     def fdiff(self, argindex=1):
         if argindex == 1:
             return cos(self.args[0])
@@ -222,13 +253,36 @@ class sin(TrigonometricFunction):
 
     @classmethod
     def eval(cls, arg):
+        from sympy.calculus import AccumBounds
         if arg.is_Number:
             if arg is S.NaN:
                 return S.NaN
             elif arg is S.Zero:
                 return S.Zero
             elif arg is S.Infinity or arg is S.NegativeInfinity:
-                return
+                return AccumBounds(-1, 1)
+
+        if isinstance(arg, AccumBounds):
+            min, max = arg.min, arg.max
+            d = floor(min/(2*S.Pi))
+            if min is not S.NegativeInfinity:
+                min = min - d*2*S.Pi
+            if max is not S.Infinity:
+                max = max - d*2*S.Pi
+            if AccumBounds(min, max).intersection(FiniteSet(S.Pi/2, 5*S.Pi/2)) \
+                    is not S.EmptySet and \
+                    AccumBounds(min, max).intersection(FiniteSet(3*S.Pi/2,
+                        7*S.Pi/2)) is not S.EmptySet:
+                return AccumBounds(-1, 1)
+            elif AccumBounds(min, max).intersection(FiniteSet(S.Pi/2, 5*S.Pi/2)) \
+                    is not S.EmptySet:
+                return AccumBounds(Min(sin(min), sin(max)), 1)
+            elif AccumBounds(min, max).intersection(FiniteSet(3*S.Pi/2, 8*S.Pi/2)) \
+                        is not S.EmptySet:
+                return AccumBounds(-1, Max(sin(min), sin(max)))
+            else:
+                return AccumBounds(Min(sin(min), sin(max)),
+                                Max(sin(min), sin(max)))
 
         if arg.could_extract_minus_sign():
             return -cls(-arg)
@@ -321,7 +375,7 @@ class sin(TrigonometricFunction):
             return I*x**-I / 2 - I*x**I /2
 
     def _eval_rewrite_as_cos(self, arg):
-        return -cos(arg + S.Pi/2)
+        return cos(arg - S.Pi / 2, evaluate=False)
 
     def _eval_rewrite_as_tan(self, arg):
         tan_half = tan(S.Half*arg)
@@ -343,6 +397,9 @@ class sin(TrigonometricFunction):
     def _eval_rewrite_as_csc(self, arg):
         return 1/csc(arg)
 
+    def _eval_rewrite_as_sec(self, arg):
+        return 1 / sec(arg - S.Pi / 2, evaluate=False)
+
     def _eval_conjugate(self):
         return self.func(self.args[0].conjugate())
 
@@ -441,6 +498,9 @@ class cos(TrigonometricFunction):
     .. [3] http://functions.wolfram.com/ElementaryFunctions/Cos
     """
 
+    def period(self, symbol=None):
+        return self._period(2*pi, symbol)
+
     def fdiff(self, argindex=1):
         if argindex == 1:
             return -sin(self.args[0])
@@ -450,21 +510,21 @@ class cos(TrigonometricFunction):
     @classmethod
     def eval(cls, arg):
         from sympy.functions.special.polynomials import chebyshevt
+        from sympy.calculus.util import AccumBounds
         if arg.is_Number:
             if arg is S.NaN:
                 return S.NaN
             elif arg is S.Zero:
                 return S.One
             elif arg is S.Infinity or arg is S.NegativeInfinity:
-                # In this cases, it is unclear if we should
-                # return S.NaN or leave un-evaluated.  One
-                # useful test case is how "limit(sin(x)/x,x,oo)"
-                # is handled.
-                # See test_sin_cos_with_infinity() an
-                # Test for issue 3308
-                # https://github.com/sympy/sympy/issues/5196
-                # For now, we return un-evaluated.
-                return
+                # In this case it is better to return AccumBounds(-1, 1)
+                # rather than returning S.NaN, since AccumBounds(-1, 1)
+                # preserves the information that sin(oo) is between
+                # -1 and 1, where S.NaN does not do that.
+                return AccumBounds(-1, 1)
+
+        if isinstance(arg, AccumBounds):
+            return sin(arg + S.Pi/2)
 
         if arg.could_extract_minus_sign():
             return cls(-arg)
@@ -600,7 +660,7 @@ class cos(TrigonometricFunction):
             return x**I/2 + x**-I/2
 
     def _eval_rewrite_as_sin(self, arg):
-        return sin(arg + S.Pi/2)
+        return sin(arg + S.Pi / 2, evaluate=False)
 
     def _eval_rewrite_as_tan(self, arg):
         tan_half = tan(S.Half*arg)**2
@@ -757,6 +817,9 @@ class cos(TrigonometricFunction):
     def _eval_rewrite_as_sec(self, arg):
         return 1/sec(arg)
 
+    def _eval_rewrite_as_csc(self, arg):
+        return 1 / sec(arg)._eval_rewrite_as_csc(arg)
+
     def _eval_conjugate(self):
         return self.func(self.args[0].conjugate())
 
@@ -841,6 +904,9 @@ class tan(TrigonometricFunction):
     .. [3] http://functions.wolfram.com/ElementaryFunctions/Tan
     """
 
+    def period(self, symbol=None):
+        return self._period(pi, symbol)
+
     def fdiff(self, argindex=1):
         if argindex == 1:
             return S.One + self**2
@@ -855,11 +921,26 @@ class tan(TrigonometricFunction):
 
     @classmethod
     def eval(cls, arg):
+        from sympy.calculus.util import AccumBounds
         if arg.is_Number:
             if arg is S.NaN:
                 return S.NaN
             elif arg is S.Zero:
                 return S.Zero
+            elif arg is S.Infinity or arg is S.NegativeInfinity:
+                return AccumBounds(S.NegativeInfinity, S.Infinity)
+
+        if isinstance(arg, AccumBounds):
+            min, max = arg.min, arg.max
+            d = floor(min/S.Pi)
+            if min is not S.NegativeInfinity:
+                min = min - d*S.Pi
+            if max is not S.Infinity:
+                max = max - d*S.Pi
+            if AccumBounds(min, max).intersection(FiniteSet(S.Pi/2, 3*S.Pi/2)):
+                return AccumBounds(S.NegativeInfinity, S.Infinity)
+            else:
+                return AccumBounds(tan(min), tan(max))
 
         if arg.could_extract_minus_sign():
             return -cls(-arg)
@@ -1024,7 +1105,7 @@ class tan(TrigonometricFunction):
         return 2*sin(x)**2/sin(2*x)
 
     def _eval_rewrite_as_cos(self, x):
-        return -cos(x + S.Pi/2)/cos(x)
+        return cos(x - S.Pi / 2, evaluate=False) / cos(x)
 
     def _eval_rewrite_as_sincos(self, arg):
         return sin(arg)/cos(arg)
@@ -1032,6 +1113,16 @@ class tan(TrigonometricFunction):
     def _eval_rewrite_as_cot(self, arg):
         return 1/cot(arg)
 
+    def _eval_rewrite_as_sec(self, arg):
+        sin_in_sec_form = sin(arg)._eval_rewrite_as_sec(arg)
+        cos_in_sec_form = cos(arg)._eval_rewrite_as_sec(arg)
+        return sin_in_sec_form / cos_in_sec_form
+
+    def _eval_rewrite_as_csc(self, arg):
+        sin_in_csc_form = sin(arg)._eval_rewrite_as_csc(arg)
+        cos_in_csc_form = cos(arg)._eval_rewrite_as_csc(arg)
+        return sin_in_csc_form / cos_in_csc_form
+
     def _eval_rewrite_as_pow(self, arg):
         y = self.rewrite(cos).rewrite(pow)
         if y.has(cos):
@@ -1100,6 +1191,9 @@ class cot(TrigonometricFunction):
     .. [3] http://functions.wolfram.com/ElementaryFunctions/Cot
     """
 
+    def period(self, symbol=None):
+        return self._period(pi, symbol)
+
     def fdiff(self, argindex=1):
         if argindex == 1:
             return S.NegativeOne - self**2
@@ -1114,12 +1208,16 @@ class cot(TrigonometricFunction):
 
     @classmethod
     def eval(cls, arg):
+        from sympy.calculus.util import AccumBounds
         if arg.is_Number:
             if arg is S.NaN:
                 return S.NaN
             if arg is S.Zero:
                 return S.ComplexInfinity
 
+        if isinstance(arg, AccumBounds):
+            return -tan(arg + S.Pi/2)
+
         if arg.could_extract_minus_sign():
             return -cls(-arg)
 
@@ -1256,7 +1354,7 @@ class cot(TrigonometricFunction):
         return 2*sin(2*x)/sin(x)**2
 
     def _eval_rewrite_as_cos(self, x):
-        return -cos(x)/cos(x + S.Pi/2)
+        return cos(x) / cos(x - S.Pi / 2, evaluate=False)
 
     def _eval_rewrite_as_sincos(self, arg):
         return cos(arg)/sin(arg)
@@ -1264,6 +1362,16 @@ class cot(TrigonometricFunction):
     def _eval_rewrite_as_tan(self, arg):
         return 1/tan(arg)
 
+    def _eval_rewrite_as_sec(self, arg):
+        cos_in_sec_form = cos(arg)._eval_rewrite_as_sec(arg)
+        sin_in_sec_form = sin(arg)._eval_rewrite_as_sec(arg)
+        return cos_in_sec_form / sin_in_sec_form
+
+    def _eval_rewrite_as_csc(self, arg):
+        cos_in_csc_form = cos(arg)._eval_rewrite_as_csc(arg)
+        sin_in_csc_form = sin(arg)._eval_rewrite_as_csc(arg)
+        return cos_in_csc_form / sin_in_csc_form
+
     def _eval_rewrite_as_pow(self, arg):
         y = self.rewrite(cos).rewrite(pow)
         if y.has(cos):
@@ -1389,6 +1497,10 @@ class ReciprocalTrigonometricFunction(TrigonometricFunction):
         if t != None and t != self._reciprocal_of(arg):
             return 1/t
 
+    def _period(self, symbol):
+        f = self.args[0]
+        return self._reciprocal_of(f).period(symbol)
+
     def fdiff(self, argindex=1):
         return -self._calculate_reciprocal("fdiff", argindex)/self**2
 
@@ -1474,6 +1586,9 @@ class sec(ReciprocalTrigonometricFunction):
     _reciprocal_of = cos
     _is_even = True
 
+    def period(self, symbol=None):
+        return self._period(symbol)
+
     def _eval_rewrite_as_cot(self, arg):
         cot_half_sq = cot(arg/2)**2
         return (cot_half_sq + 1)/(cot_half_sq - 1)
@@ -1484,6 +1599,15 @@ class sec(ReciprocalTrigonometricFunction):
     def _eval_rewrite_as_sincos(self, arg):
         return sin(arg)/(cos(arg)*sin(arg))
 
+    def _eval_rewrite_as_sin(self, arg):
+        return (1 / cos(arg)._eval_rewrite_as_sin(arg))
+
+    def _eval_rewrite_as_tan(self, arg):
+        return (1 / cos(arg)._eval_rewrite_as_tan(arg))
+
+    def _eval_rewrite_as_csc(self, arg):
+        return csc(pi / 2 - arg, evaluate=False)
+
     def fdiff(self, argindex=1):
         if argindex == 1:
             return tan(self.args[0])*sec(self.args[0])
@@ -1542,6 +1666,9 @@ class csc(ReciprocalTrigonometricFunction):
     _reciprocal_of = sin
     _is_odd = True
 
+    def period(self, symbol=None):
+        return self._period(symbol)
+
     def _eval_rewrite_as_sin(self, arg):
         return (1/sin(arg))
 
@@ -1552,6 +1679,15 @@ class csc(ReciprocalTrigonometricFunction):
         cot_half = cot(arg/2)
         return (1 + cot_half**2)/(2*cot_half)
 
+    def _eval_rewrite_as_cos(self, arg):
+        return (1 / sin(arg)._eval_rewrite_as_cos(arg))
+
+    def _eval_rewrite_as_sec(self, arg):
+        return sec(pi / 2 - arg, evaluate=False)
+
+    def _eval_rewrite_as_tan(self, arg):
+        return (1 / sin(arg)._eval_rewrite_as_tan(arg))
+
     def fdiff(self, argindex=1):
         if argindex == 1:
             return -cot(self.args[0])*csc(self.args[0])
@@ -2279,7 +2415,7 @@ class acot(InverseTrigonometricFunction):
 
 
 class asec(InverseTrigonometricFunction):
-    """
+    r"""
     The inverse secant function.
 
     Returns the arc secant of x (measured in radians).
