@@ -1,6 +1,7 @@
-from sympy import (diff, trigsimp, expand, sin, cos, solve, Symbol, sympify,
-                   eye, symbols, Dummy, ImmutableMatrix as Matrix)
-from sympy.core.compatibility import string_types, u, range
+from sympy.core.backend import (diff, expand, sin, cos, sympify,
+                   eye, symbols, ImmutableMatrix as Matrix, MatrixBase)
+from sympy import (trigsimp, solve, Symbol, Dummy)
+from sympy.core.compatibility import string_types, range
 from sympy.physics.vector.vector import Vector, _check_vector
 
 __all__ = ['CoordinateSym', 'ReferenceFrame']
@@ -45,7 +46,11 @@ class CoordinateSym(Symbol):
     """
 
     def __new__(cls, name, frame, index):
-        obj = super(CoordinateSym, cls).__new__(cls, name)
+        # We can't use the cached Symbol.__new__ because this class depends on
+        # frame and index, which are not passed to Symbol.__xnew__.
+        assumptions = {}
+        super(CoordinateSym, cls)._sanitize(assumptions, cls)
+        obj = super(CoordinateSym, cls).__xnew__(cls, name, **assumptions)
         _check_frame(frame)
         if index not in range(0, 3):
             raise ValueError("Invalid index specified")
@@ -84,6 +89,7 @@ class ReferenceFrame(object):
     vector, defined in another frame.
 
     """
+    _count = 0
 
     def __init__(self, name, indices=None, latexs=None, variables=None):
         """ReferenceFrame initialization method.
@@ -137,9 +143,9 @@ class ReferenceFrame(object):
             self.str_vecs = [(name + '[\'' + indices[0] + '\']'),
                              (name + '[\'' + indices[1] + '\']'),
                              (name + '[\'' + indices[2] + '\']')]
-            self.pretty_vecs = [(name.lower() + u("_") + indices[0]),
-                                (name.lower() + u("_") + indices[1]),
-                                (name.lower() + u("_") + indices[2])]
+            self.pretty_vecs = [(name.lower() + u"_" + indices[0]),
+                                (name.lower() + u"_" + indices[1]),
+                                (name.lower() + u"_" + indices[2])]
             self.latex_vecs = [(r"\mathbf{\hat{%s}_{%s}}" % (name.lower(),
                                indices[0])), (r"\mathbf{\hat{%s}_{%s}}" %
                                (name.lower(), indices[1])),
@@ -149,9 +155,9 @@ class ReferenceFrame(object):
         # Second case, when no custom indices are supplied
         else:
             self.str_vecs = [(name + '.x'), (name + '.y'), (name + '.z')]
-            self.pretty_vecs = [name.lower() + u("_x"),
-                                name.lower() + u("_y"),
-                                name.lower() + u("_z")]
+            self.pretty_vecs = [name.lower() + u"_x",
+                                name.lower() + u"_y",
+                                name.lower() + u"_z"]
             self.latex_vecs = [(r"\mathbf{\hat{%s}_x}" % name.lower()),
                                (r"\mathbf{\hat{%s}_y}" % name.lower()),
                                (r"\mathbf{\hat{%s}_z}" % name.lower())]
@@ -194,6 +200,8 @@ class ReferenceFrame(object):
         self.varlist = (CoordinateSym(variables[0], self, 0), \
                         CoordinateSym(variables[1], self, 1), \
                         CoordinateSym(variables[2], self, 2))
+        ReferenceFrame._count += 1
+        self.index = ReferenceFrame._count
 
     def __getitem__(self, ind):
         """
@@ -421,10 +429,12 @@ class ReferenceFrame(object):
             defined in relation to.
         rot_type : str
             The type of orientation matrix that is being created. Supported
-            types are 'Body', 'Space', 'Quaternion', and 'Axis'. See examples
-            for correct usage.
+            types are 'Body', 'Space', 'Quaternion', 'Axis', and 'DCM'.
+            See examples for correct usage.
         amounts : list OR value
             The quantities that the orientation matrix will be defined by.
+            In case of rot_type='DCM', value must be a
+            sympy.matrices.MatrixBase object (or subclasses of it).
         rot_order : str
             If applicable, the order of a series of rotations.
 
@@ -432,8 +442,8 @@ class ReferenceFrame(object):
         ========
 
         >>> from sympy.physics.vector import ReferenceFrame, Vector
-        >>> from sympy import symbols
-        >>> q0, q1, q2, q3, q4 = symbols('q0 q1 q2 q3 q4')
+        >>> from sympy import symbols, eye, ImmutableMatrix
+        >>> q0, q1, q2, q3 = symbols('q0 q1 q2 q3')
         >>> N = ReferenceFrame('N')
         >>> B = ReferenceFrame('B')
 
@@ -464,20 +474,34 @@ class ReferenceFrame(object):
 
         >>> B.orient(N, 'Quaternion', [q0, q1, q2, q3])
 
-        Last is Axis. This is a rotation about an arbitrary, non-time-varying
+        Next is Axis. This is a rotation about an arbitrary, non-time-varying
         axis by some angle. The axis is supplied as a Vector. This is how
         simple rotations are defined.
 
         >>> B.orient(N, 'Axis', [q1, N.x + 2 * N.y])
 
+        Last is DCM (Direction Cosine Matrix). This is a rotation matrix
+        given manually.
+
+        >>> B.orient(N, 'DCM', eye(3))
+        >>> B.orient(N, 'DCM', ImmutableMatrix([[0, 1, 0], [0, 0, -1], [-1, 0, 0]]))
+
         """
 
         from sympy.physics.vector.functions import dynamicsymbols
         _check_frame(parent)
-        amounts = list(amounts)
-        for i, v in enumerate(amounts):
-            if not isinstance(v, Vector):
-                amounts[i] = sympify(v)
+
+        # Allow passing a rotation matrix manually.
+        if rot_type == 'DCM':
+            # When rot_type == 'DCM', then amounts must be a Matrix type object
+            # (e.g. sympy.matrices.dense.MutableDenseMatrix).
+            if not isinstance(amounts, MatrixBase):
+                raise TypeError("Amounts must be a sympy Matrix type object.")
+        else:
+            amounts = list(amounts)
+            for i, v in enumerate(amounts):
+                if not isinstance(v, Vector):
+                    amounts[i] = sympify(v)
 
         def _rot(axis, angle):
             """DCM for simple axis 1,2,or 3 rotations. """
@@ -549,15 +573,23 @@ class ReferenceFrame(object):
             a3 = int(rot_order[2])
             parent_orient = (_rot(a3, amounts[2]) * _rot(a2, amounts[1])
                     * _rot(a1, amounts[0]))
+        elif rot_type == 'DCM':
+            parent_orient = amounts
         else:
             raise NotImplementedError('That is not an implemented rotation')
         #Reset the _dcm_cache of this frame, and remove it from the _dcm_caches
         #of the frames it is linked to. Also remove it from the _dcm_dict of
         #its parent
         frames = self._dcm_cache.keys()
+        dcm_dict_del = []
+        dcm_cache_del = []
         for frame in frames:
             if frame in self._dcm_dict:
-                del frame._dcm_dict[self]
+                dcm_dict_del += [frame]
+            dcm_cache_del += [frame]
+        for frame in dcm_dict_del:
+            del frame._dcm_dict[self]
+        for frame in dcm_cache_del:
             del frame._dcm_cache[self]
         #Add the dcm relationship to _dcm_dict
         self._dcm_dict = self._dlist[0] = {}
@@ -581,6 +613,8 @@ class ReferenceFrame(object):
         elif rot_type == 'AXIS':
             thetad = (amounts[0]).diff(dynamicsymbols._t)
             wvec = thetad * amounts[1].express(parent).normalize()
+        elif rot_type == 'DCM':
+            wvec = self._w_diff_dcm(parent)
         else:
             try:
                 from sympy.polys.polyerrors import CoercionFailed
@@ -601,8 +635,8 @@ class ReferenceFrame(object):
         parent._ang_vel_dict.update({self: -wvec})
         self._var_dict = {}
 
-    def orientnew(self, newname, rot_type, amounts, rot_order='', variables=None,
-                  indices=None, latexs=None):
+    def orientnew(self, newname, rot_type, amounts, rot_order='',
+                  variables=None, indices=None, latexs=None):
         """Creates a new ReferenceFrame oriented with respect to this Frame.
 
         See ReferenceFrame.orient() for acceptable rotation types, amounts,
@@ -620,28 +654,53 @@ class ReferenceFrame(object):
         rot_order : str
             If applicable, the order of a series of rotations.
 
-
         Examples
         ========
 
         >>> from sympy.physics.vector import ReferenceFrame, Vector
         >>> from sympy import symbols
-        >>> q1 = symbols('q1')
+        >>> q0, q1, q2, q3 = symbols('q0 q1 q2 q3')
         >>> N = ReferenceFrame('N')
+
+        Now we have a choice of how to implement the orientation. First is
+        Body. Body orientation takes this reference frame through three
+        successive simple rotations. Acceptable rotation orders are of length
+        3, expressed in XYZ or 123, and cannot have a rotation about about an
+        axis twice in a row.
+
+        >>> A = N.orientnew('A', 'Body', [q1, q2, q3], '123')
+        >>> A = N.orientnew('A', 'Body', [q1, q2, 0], 'ZXZ')
+        >>> A = N.orientnew('A', 'Body', [0, 0, 0], 'XYX')
+
+        Next is Space. Space is like Body, but the rotations are applied in the
+        opposite order.
+
+        >>> A = N.orientnew('A', 'Space', [q1, q2, q3], '312')
+
+        Next is Quaternion. This orients the new ReferenceFrame with
+        Quaternions, defined as a finite rotation about lambda, a unit vector,
+        by some amount theta.
+        This orientation is described by four parameters:
+        q0 = cos(theta/2)
+        q1 = lambda_x sin(theta/2)
+        q2 = lambda_y sin(theta/2)
+        q3 = lambda_z sin(theta/2)
+        Quaternion does not take in a rotation order.
+
+        >>> A = N.orientnew('A', 'Quaternion', [q0, q1, q2, q3])
+
+        Last is Axis. This is a rotation about an arbitrary, non-time-varying
+        axis by some angle. The axis is supplied as a Vector. This is how
+        simple rotations are defined.
+
         >>> A = N.orientnew('A', 'Axis', [q1, N.x])
 
-
-        .orient() documentation:\n
-        ========================
-
         """
 
         newframe = self.__class__(newname, variables, indices, latexs)
         newframe.orient(self, rot_type, amounts, rot_order)
         return newframe
 
-    orientnew.__doc__ += orient.__doc__
-
     def set_ang_acc(self, otherframe, value):
         """Define the angular acceleration Vector in a ReferenceFrame.
 
@@ -729,6 +788,46 @@ class ReferenceFrame(object):
         """The basis Vector for the ReferenceFrame, in the z direction. """
         return self._z
 
+    def partial_velocity(self, frame, *gen_speeds):
+        """Returns the partial angular velocities of this frame in the given
+        frame with respect to one or more provided generalized speeds.
+
+        Parameters
+        ==========
+        frame : ReferenceFrame
+            The frame with which the angular velocity is defined in.
+        gen_speeds : functions of time
+            The generalized speeds.
+
+        Returns
+        =======
+        partial_velocities : tuple of Vector
+            The partial angular velocity vectors corresponding to the provided
+            generalized speeds.
+
+        Examples
+        ========
+
+        >>> from sympy.physics.vector import ReferenceFrame, dynamicsymbols
+        >>> N = ReferenceFrame('N')
+        >>> A = ReferenceFrame('A')
+        >>> u1, u2 = dynamicsymbols('u1, u2')
+        >>> A.set_ang_vel(N, u1 * A.x + u2 * N.y)
+        >>> A.partial_velocity(N, u1)
+        A.x
+        >>> A.partial_velocity(N, u1, u2)
+        (A.x, N.y)
+
+        """
+
+        partials = [self.ang_vel_in(frame).diff(speed, frame, var_in_dcm=False)
+                    for speed in gen_speeds]
+
+        if len(partials) == 1:
+            return partials[0]
+        else:
+            return tuple(partials)
+
 
 def _check_frame(other):
     from .vector import VectorTypeError
