@@ -2,12 +2,13 @@
 
 from __future__ import print_function, division
 
-from sympy.polys.polyerrors import PolynomialError, GeneratorsNeeded, GeneratorsError
+from sympy.polys.polyerrors import PolynomialError, GeneratorsError
 from sympy.polys.polyoptions import build_options
 
 from sympy.core.exprtools import decompose_power, decompose_power_rat
 
-from sympy.core import S, Add, Mul, Pow, expand_mul, expand_multinomial
+from sympy.core import (S, Add, Mul, Pow, Expr,
+    expand_mul, expand_multinomial)
 
 from sympy.core.compatibility import range
 
@@ -269,14 +270,6 @@ def _parallel_dict_from_expr_no_gens(exprs, opt):
 
         reprs.append(terms)
 
-    if not gens:
-        if len(exprs) == 1:
-            arg = exprs[0]
-        else:
-            arg = (exprs,)
-
-        raise GeneratorsNeeded("specify generators to give %s a meaning" % arg)
-
     gens = _sort_gens(gens, opt=opt)
     k, indices = len(gens), {}
 
@@ -356,10 +349,9 @@ def _dict_from_expr(expr, opt):
                 and expr.base.is_Add)
 
     if opt.expand is not False:
-        try:
-            expr = expr.expand()
-        except AttributeError:
-            raise PolynomialError('expression must support expand method')
+        if not isinstance(expr, Expr):
+            raise PolynomialError('expression must be of type Expr')
+        expr = expr.expand()
         # TODO: Integrate this into expand() itself
         while any(_is_expandable_pow(i) or i.is_Mul and
             any(_is_expandable_pow(j) for j in i.args) for i in
