@@ -15,11 +15,15 @@ from sympy.polys.monomials import MonomialOps
 from sympy.polys.orderings import lex
 from sympy.polys.heuristicgcd import heugcd
 from sympy.polys.compatibility import IPolys
-from sympy.polys.polyutils import expr_from_dict, _dict_reorder, _parallel_dict_from_expr
-from sympy.polys.polyerrors import CoercionFailed, GeneratorsError, GeneratorsNeeded, ExactQuotientFailed, MultivariatePolynomialError
+from sympy.polys.polyutils import (expr_from_dict, _dict_reorder,
+                                   _parallel_dict_from_expr)
+from sympy.polys.polyerrors import (
+    CoercionFailed, GeneratorsError,
+    ExactQuotientFailed, MultivariatePolynomialError)
 from sympy.polys.domains.domainelement import DomainElement
 from sympy.polys.domains.polynomialring import PolynomialRing
-from sympy.polys.polyoptions import Domain as DomainOpt, Order as OrderOpt, build_options
+from sympy.polys.polyoptions import (Domain as DomainOpt,
+                                     Order as OrderOpt, build_options)
 from sympy.polys.densebasic import dmp_to_dict, dmp_from_dict
 from sympy.polys.constructor import construct_domain
 from sympy.printing.defaults import DefaultPrinting
@@ -167,11 +171,8 @@ def sring(exprs, *symbols, **options):
         return (_ring, polys)
 
 def _parse_symbols(symbols):
-    if not symbols:
-        raise GeneratorsNeeded("generators weren't specified")
-
     if isinstance(symbols, string_types):
-        return _symbols(symbols, seq=True)
+        return _symbols(symbols, seq=True) if symbols else ()
     elif isinstance(symbols, Expr):
         return (symbols,)
     elif is_sequence(symbols):
@@ -193,15 +194,16 @@ class PolyRing(DefaultPrinting, IPolys):
         domain = DomainOpt.preprocess(domain)
         order = OrderOpt.preprocess(order)
 
-        _hash = hash((cls.__name__, symbols, ngens, domain, order))
-        obj = _ring_cache.get(_hash)
+        _hash_tuple = (cls.__name__, symbols, ngens, domain, order)
+        obj = _ring_cache.get(_hash_tuple)
 
         if obj is None:
             if domain.is_Composite and set(symbols) & set(domain.symbols):
                 raise GeneratorsError("polynomial ring and it's ground domain share generators")
 
             obj = object.__new__(cls)
-            obj._hash = _hash
+            obj._hash_tuple = _hash_tuple
+            obj._hash = hash(_hash_tuple)
             obj.dtype = type("PolyElement", (PolyElement,), {"ring": obj})
             obj.symbols = symbols
             obj.ngens = ngens
@@ -214,14 +216,26 @@ class PolyRing(DefaultPrinting, IPolys):
 
             obj._one = [(obj.zero_monom, domain.one)]
 
-            codegen = MonomialOps(ngens)
-            obj.monomial_mul = codegen.mul()
-            obj.monomial_pow = codegen.pow()
-            obj.monomial_mulpow = codegen.mulpow()
-            obj.monomial_ldiv = codegen.ldiv()
-            obj.monomial_div = codegen.div()
-            obj.monomial_lcm = codegen.lcm()
-            obj.monomial_gcd = codegen.gcd()
+            if ngens:
+                # These expect monomials in at least one variable
+                codegen = MonomialOps(ngens)
+                obj.monomial_mul = codegen.mul()
+                obj.monomial_pow = codegen.pow()
+                obj.monomial_mulpow = codegen.mulpow()
+                obj.monomial_ldiv = codegen.ldiv()
+                obj.monomial_div = codegen.div()
+                obj.monomial_lcm = codegen.lcm()
+                obj.monomial_gcd = codegen.gcd()
+            else:
+                monunit = lambda a, b: ()
+                obj.monomial_mul = monunit
+                obj.monomial_pow = monunit
+                obj.monomial_mulpow = lambda a, b, c: ()
+                obj.monomial_ldiv = monunit
+                obj.monomial_div = monunit
+                obj.monomial_lcm = monunit
+                obj.monomial_gcd = monunit
+
 
             if order is lex:
                 obj.leading_expv = lambda f: max(f)
@@ -235,7 +249,7 @@ class PolyRing(DefaultPrinting, IPolys):
                     if not hasattr(obj, name):
                         setattr(obj, name, generator)
 
-            _ring_cache[_hash] = obj
+            _ring_cache[_hash_tuple] = obj
 
         return obj
 
@@ -267,10 +281,12 @@ class PolyRing(DefaultPrinting, IPolys):
         return self._hash
 
     def __eq__(self, other):
-        return self is other
+        return isinstance(other, PolyRing) and \
+            (self.symbols, self.domain, self.ngens, self.order) == \
+            (other.symbols, other.domain, other.ngens, other.order)
 
     def __ne__(self, other):
-        return self is not other
+        return not self.__eq__(other)
 
     def clone(self, symbols=None, domain=None, order=None):
         return self.__class__(symbols or self.symbols, domain or self.domain, order or self.order)
@@ -375,7 +391,10 @@ class PolyRing(DefaultPrinting, IPolys):
     def index(self, gen):
         """Compute index of ``gen`` in ``self.gens``. """
         if gen is None:
-            i = 0
+            if self.ngens:
+                i = 0
+            else:
+                i = -1  # indicate impossible choice
         elif isinstance(gen, int):
             i = gen
 
@@ -594,12 +613,12 @@ class PolyElement(DomainElement, DefaultPrinting, CantSympify, dict):
 
     def as_expr_dict(self):
         to_sympy = self.ring.domain.to_sympy
-        return dict([ (monom, to_sympy(coeff)) for monom, coeff in self.iterterms() ])
+        return {monom: to_sympy(coeff) for monom, coeff in self.iterterms()}
 
     def clear_denoms(self):
         domain = self.ring.domain
 
-        if not domain.has_Field or not domain.has_assoc_Ring:
+        if not domain.is_Field or not domain.has_assoc_Ring:
             return domain.one, self
 
         ground_ring = domain.get_ring()
@@ -638,7 +657,7 @@ class PolyElement(DomainElement, DefaultPrinting, CantSympify, dict):
         """
         if not p2:
             return not p1
-        elif isinstance(p2, p1.ring.dtype):
+        elif isinstance(p2, PolyElement) and p2.ring == p1.ring:
             return dict.__eq__(p1, p2)
         elif len(p1) > 1:
             return False
@@ -758,6 +777,7 @@ class PolyElement(DomainElement, DefaultPrinting, CantSympify, dict):
         if not self:
             return printer._print(self.ring.domain.zero)
         prec_add = precedence["Add"]
+        prec_mul = precedence["Mul"]
         prec_atom = precedence["Atom"]
         ring = self.ring
         symbols = ring.symbols
@@ -776,7 +796,7 @@ class PolyElement(DomainElement, DefaultPrinting, CantSympify, dict):
                 if not positive:
                     coeff = -coeff
                 if coeff != 1:
-                    scoeff = printer.parenthesize(coeff, prec_add)
+                    scoeff = printer.parenthesize(coeff, prec_mul, strict=True)
                 else:
                     scoeff = ''
             sexpv = []
@@ -784,10 +804,10 @@ class PolyElement(DomainElement, DefaultPrinting, CantSympify, dict):
                 exp = expv[i]
                 if not exp:
                     continue
-                symbol = printer.parenthesize(symbols[i], prec_atom-1)
+                symbol = printer.parenthesize(symbols[i], prec_atom, strict=True)
                 if exp != 1:
                     if exp != int(exp) or exp < 0:
-                        sexp = printer.parenthesize(exp, prec_atom)
+                        sexp = printer.parenthesize(exp, prec_atom, strict=False)
                     else:
                         sexp = exp
                     sexpv.append(exp_pattern % (symbol, sexp))
@@ -860,10 +880,14 @@ class PolyElement(DomainElement, DefaultPrinting, CantSympify, dict):
 
     @property
     def is_squarefree(f):
+        if not f.ring.ngens:
+            return True
         return f.ring.dmp_sqf_p(f)
 
     @property
     def is_irreducible(f):
+        if not f.ring.ngens:
+            return True
         return f.ring.dmp_irreducible_p(f)
 
     @property
@@ -1334,7 +1358,7 @@ class PolyElement(DomainElement, DefaultPrinting, CantSympify, dict):
         domain_quo = domain.quo
         monomial_div = self.ring.monomial_div
 
-        if domain.has_Field:
+        if domain.is_Field:
             def term_div(a_lm_a_lc, b_lm_b_lc):
                 a_lm, a_lc = a_lm_a_lc
                 b_lm, b_lc = b_lm_b_lc
@@ -1587,6 +1611,8 @@ class PolyElement(DomainElement, DefaultPrinting, CantSympify, dict):
 
         if not f:
             return -oo
+        elif i < 0:
+            return 0
         else:
             return max([ monom[i] for monom in f.itermonoms() ])
 
@@ -1613,6 +1639,8 @@ class PolyElement(DomainElement, DefaultPrinting, CantSympify, dict):
 
         if not f:
             return -oo
+        elif i < 0:
+            return 0
         else:
             return min([ monom[i] for monom in f.itermonoms() ])
 
@@ -1748,7 +1776,7 @@ class PolyElement(DomainElement, DefaultPrinting, CantSympify, dict):
         """
         p = self.ring.zero
         expv = self.leading_expv()
-        if expv:
+        if expv is not None:
             p[expv] = self[expv]
         return p
 
@@ -1952,7 +1980,7 @@ class PolyElement(DomainElement, DefaultPrinting, CantSympify, dict):
         if not f or x == domain.one:
             return f
 
-        if domain.has_Field:
+        if domain.is_Field:
             quo = domain.quo
             terms = [ (monom, quo(coeff, x)) for monom, coeff in f.iterterms() ]
         else:
@@ -2066,14 +2094,14 @@ class PolyElement(DomainElement, DefaultPrinting, CantSympify, dict):
         f = self
         domain = f.ring.domain
 
-        if not domain.has_Field:
+        if not domain.is_Field:
             fc, f = f.primitive()
             gc, g = g.primitive()
             c = domain.lcm(fc, gc)
 
         h = (f*g).quo(f.gcd(g))
 
-        if not domain.has_Field:
+        if not domain.is_Field:
             return h.mul_ground(c)
         else:
             return h.monic()
@@ -2182,7 +2210,7 @@ class PolyElement(DomainElement, DefaultPrinting, CantSympify, dict):
 
         domain = ring.domain
 
-        if not (domain.has_Field and domain.has_assoc_Ring):
+        if not (domain.is_Field and domain.has_assoc_Ring):
             _, p, q = f.cofactors(g)
 
             if q.is_negative:
@@ -2239,7 +2267,7 @@ class PolyElement(DomainElement, DefaultPrinting, CantSympify, dict):
         for expv, coeff in f.iterterms():
             if expv[i]:
                 e = ring.monomial_ldiv(expv, m)
-                g[e] = coeff*expv[i]
+                g[e] = ring.domain_new(coeff*expv[i])
         return g
 
     def __call__(f, *values):
