@@ -1,8 +1,8 @@
-from sympy import sin, cos, atan2, log, exp, gamma, conjugate, sqrt, \
-    factorial, Integral, Piecewise, Add, diff, symbols, S, Float, Dummy, Eq
-from sympy import Catalan, EulerGamma, E, GoldenRatio, I, pi
-from sympy import Function, Rational, Integer, Lambda
+from sympy import (sin, cos, atan2, log, exp, gamma, conjugate, sqrt,
+    factorial, Integral, Piecewise, Add, diff, symbols, S, Float, Dummy, Eq,
+    Range, Catalan, EulerGamma, E, GoldenRatio, I, pi, Function, Rational, Integer, Lambda, sign)
 
+from sympy.codegen import For, Assignment
 from sympy.core.relational import Relational
 from sympy.logic.boolalg import And, Or, Not, Equivalent, Xor
 from sympy.printing.fcode import fcode, FCodePrinter
@@ -22,6 +22,16 @@ def test_printmethod():
     assert fcode(nint(x)) == "      nint(x)"
 
 
+def test_fcode_sign():  #issue 12267
+    x=symbols('x')
+    y=symbols('y', integer=True)
+    z=symbols('z', complex=True)
+    assert fcode(sign(x), standard=95, source_format='free') == "merge(0d0, dsign(1d0, x), x == 0d0)"
+    assert fcode(sign(y), standard=95, source_format='free') == "merge(0, isign(1, y), y == 0)"
+    assert fcode(sign(z), standard=95, source_format='free') == "merge(cmplx(0d0, 0d0), z/abs(z), abs(z) == 0d0)"
+    raises(NotImplementedError, lambda: fcode(sign(x)))
+
+
 def test_fcode_Pow():
     x, y = symbols('x,y')
     n = symbols('n', integer=True)
@@ -343,8 +353,8 @@ def test_fcode_Xlogical():
 
 def test_fcode_Relational():
     x, y = symbols("x y")
-    assert fcode(Relational(x, y, "=="), source_format="free") == "Eq(x, y)"
-    assert fcode(Relational(x, y, "!="), source_format="free") == "Ne(x, y)"
+    assert fcode(Relational(x, y, "=="), source_format="free") == "x == y"
+    assert fcode(Relational(x, y, "!="), source_format="free") == "x /= y"
     assert fcode(Relational(x, y, ">="), source_format="free") == "x >= y"
     assert fcode(Relational(x, y, "<="), source_format="free") == "x <= y"
     assert fcode(Relational(x, y, ">"), source_format="free") == "x > y"
@@ -658,10 +668,33 @@ def test_Matrix_printing():
     assert fcode(m, M) == (
         "      M(1, 1) = sin(q(2, 1))\n"
         "      M(2, 1) = q(2, 1) + q(3, 1)\n"
-        "      M(3, 1) = 2*q(5, 1)*1.0/q(2, 1)\n"
+        "      M(3, 1) = 2*q(5, 1)/q(2, 1)\n"
         "      M(1, 2) = 0\n"
         "      M(2, 2) = q(4, 1)\n"
-        "      M(3, 2) = 4 + sqrt(q(1, 1))\n"
+        "      M(3, 2) = sqrt(q(1, 1)) + 4\n"
         "      M(1, 3) = cos(q(3, 1))\n"
         "      M(2, 3) = 5\n"
         "      M(3, 3) = 0")
+
+
+def test_fcode_For():
+    x, y = symbols('x y')
+
+    f = For(x, Range(0, 10, 2), [Assignment(y, x * y)])
+    sol = fcode(f)
+    assert sol == ("      do x = 0, 10, 2\n"
+                   "         y = x*y\n"
+                   "      end do")
+
+
+def test_MatrixElement_printing():
+    # test cases for issue #11821
+    A = MatrixSymbol("A", 1, 3)
+    B = MatrixSymbol("B", 1, 3)
+    C = MatrixSymbol("C", 1, 3)
+
+    assert(fcode(A[0, 0]) == "      A(1, 1)")
+    assert(fcode(3 * A[0, 0]) == "      3*A(1, 1)")
+
+    F = C[0, 0].subs(C, A - B)
+    assert(fcode(F) == "      ((-1)*B + A)(1, 1)")
